/**
 * Synchronize the users and devices with xMatters
 * @param caServiceDeskWS instance of the CA Service Desk Web Service object
 */
var usersProcessed;

function processCASDUsers(caServiceDeskWS, qualification, isDelete, isUpdateOnly)
{
    log.info("Enter - processCASDUsers");
    log.info("processCASDUsers. isDelete [" + isDelete + "], qualification [" + qualification + "], chunkSize [" + CHUNK_SIZE + "], isUpdateOnly [" + isUpdateOnly + "]");

    usersProcessed = 0;

    var doQueryReturn = caServiceDeskWS.doQuery(OBJECT_TYPE_CONTACT, qualification);

    var listLength = doQueryReturn.listLength;
    var listHandle = doQueryReturn.listHandle;
    var numberOfChunks = Math.ceil(listLength / CHUNK_SIZE);
    log.info("listLength [" + listLength + "], chunkSize [" + CHUNK_SIZE + "], numberOfChunks [" + numberOfChunks + "]");
    
	try
	{
		for (var i = 0; i < numberOfChunks; i++)
		{
			var startIndex = i * CHUNK_SIZE;

			var endIndex = parseInt(startIndex) + parseInt(CHUNK_SIZE);
			endIndex = (listLength < endIndex ? listLength : endIndex) - 1;

			log.info("chunk [" + i + "], startIndex [" + startIndex + "], endIndex [" + endIndex + "]");

			// Get one chunk of the list of users
			var udsObjectList = caServiceDeskWS.getListValues(listHandle, startIndex, endIndex, ATTRIBUTES_CNT_OBJECT);

			log.info("getListValues returned " + udsObjectList.length + " entries");

			// Iterate over the users in this chunk, processing them one at a time
			udsObjectList.forEach(function(udsObject){
				tryProcessOneCasdUser(udsObject, caServiceDeskWS, isDelete, isUpdateOnly);
			});
		}
	}
	finally
	{
		log.info("Releasing doQuery resources for handle: " + listHandle);
		caServiceDeskWS.freeListHandles([listHandle]);
	}

    log.info("processCASDUsers - processed " + usersProcessed + " users");
    log.info("Exit - processCASDUsers");
}


/**
 * ---------------------------------------------------------------------------------------------------------------------
 * tryProcessOneCasdUser
 *
 * Add/update or delete a single Service Desk user in xMatters. If anything goes wrong at this level, record it in the
 * failure list, but continue processing
 * ---------------------------------------------------------------------------------------------------------------------
 */
function tryProcessOneCasdUser(udsObject, caServiceDeskWS, isDelete, isUpdateOnly)
{
    log.info("Enter - tryProcessOneCasdUser");
    var user = null;

    var targetId = udsObject.userid;
    log.info("Processing user [" + targetId + "]");

    try
    {
        if (isDelete)
        {
            user = new User(targetId);
        }
        else
        {
            // Get the list of Service Desk roles associated with this person
            var casdRolesList = getUserRoles(caServiceDeskWS, udsObject);

            user = makeUserForAddUpdate(udsObject, casdRolesList, isUpdateOnly);
        }

        (new xMattersUserSync(syncList)).syncUser(user, isDelete);

        usersProcessed++;
        log.info("Finished processing user [" + targetId + "]");
    }
    catch (e)
    {
        log.error("tryProcessOneCasdUser - while processing user Service Desk Login ID [" + targetId + "] got exception [" + e.toString() + "]");
        failedList.add((isDelete ? "Delete" : "Add/Update") + " User: " + targetId + " failed to synchronize: " + e.toString());
    }

    log.info("Exit - tryProcessOneCasdUser");
    return user;
}

/**
 * Helper function which, given a web service response list of contacts creates
 * the User objects.
 * @param attributes collection of attributes from the current UDSObject Contact
 * @return Populated User Object
 */
function makeUserForAddUpdate(udsObject, casdRolesList, isUpdateOnly)
{
    log.info("Enter - makeUserForAddUpdate");

    // The userid attribute is not guaranteed to exist in CASDM
    var targetName = udsObject.userid;

    if (isEmpty(targetName))
    {
        throw "User ID not found for user with CA SDM with id U" + udsObject.id;
    }

    log.info("Create xMatters sync user object for [" + targetName + "], isUpdateOnly [" + isUpdateOnly + "]");

    var user = new User(targetName);

    user.updateOnly = isUpdateOnly;

    // Some user properties are not derived from Service Desk information. These are only given initial values when the user
    // is created. After that, the values in xMatters are preserved during updates.
    if (!user.userExists)
    {
        user.site = DEFAULT_USER_SITE;
        user.externallyOwned = EXTERNALLY_OWN_USERS;
        user.webLoginType = WEB_LOGIN_TYPE;
        user.ldapDomain = WEB_LOGIN_LDAP_DOMAIN;

        // The default supervisor is defined in configuration.js and depends on whether xMatters has
        // merged administrator accounts or not.
        user.supervisors = (new SupervisorMapper()).getSupervisorsForUser(user);
    }

    // Only set the roles array value:
    // IF the user exists and MAP_USER_ROLES = true OR the user is being added.
    // Otherwise, xMatters will be treated as the master source.
    if (!user.userExists || MAP_USER_ROLES)
    {
        user.roles = (new UserRolesMapper()).getXmUserRolesFor(casdRolesList);
    }

    // ---------------------------------------------------------------------------------------------------------
    // Transfer the basic xMatters user properties from CA SD
    // ---------------------------------------------------------------------------------------------------------

    var deleteFlag = udsObject.delete_flag;
    user.active = !isEmpty(deleteFlag) && deleteFlag == "0";

    user.firstName = isEmpty(udsObject.first_name) ? DEFAULT_USER_FIRST_NAME : udsObject.first_name;
    user.lastName = udsObject.last_name;
    user.webLogin = user.targetName

    // ---------------------------------------------------------------------------------------------------------
    // Transfer device information from Service Desk to the user object that will be loaded into xMatters.
    //
    // If the attribute associated with a given device is empty, the device is flagged for deletion in xMatters.
    // ---------------------------------------------------------------------------------------------------------

    var casdDeviceInfo = null;
    var syncDevice = null;
    var phoneNumberHelper = new PHONE_NUMBER();

    casdDeviceInfo = udsObject.alt_phone;
    syncDevice = phoneNumberHelper.setVoicePhoneOrPager(user, new VoiceDevice(user.targetName, "Home Phone"), casdDeviceInfo);
    syncDevice.externallyOwned = user.externallyOwned;
    user.devices.add(syncDevice);
    log.debug("alt_phone [" + casdDeviceInfo + "], Home Phone [" + syncDevice + "]");

    casdDeviceInfo = udsObject.email_address;
    syncDevice = new EmailDevice(user.targetName, "Work Email");
    syncDevice.isDelete = isEmpty(casdDeviceInfo);
    syncDevice.address = casdDeviceInfo;
    syncDevice.externallyOwned = user.externallyOwned;
    user.devices.add(syncDevice);
    log.debug("email_address [" + casdDeviceInfo + "], Work Email [" + syncDevice + "]");

    casdDeviceInfo = udsObject.fax_phone;
    syncDevice = phoneNumberHelper.setVoicePhoneOrPager(user, new VoiceDevice(user.targetName, "Other Phone"), casdDeviceInfo);
    syncDevice.externallyOwned = user.externallyOwned;
    user.devices.add(syncDevice);
    log.debug("fax_phone [" + casdDeviceInfo + "], Other Phone [" + syncDevice + "]");

    casdDeviceInfo = udsObject.beeper_phone;
    syncDevice = phoneNumberHelper.setVoicePhoneOrPager(user, new TextPhoneDevice(user.targetName, "SMS Phone"), casdDeviceInfo);
    syncDevice.externallyOwned = user.externallyOwned;
    user.devices.add(syncDevice);
    log.debug("beeper_phone [" + casdDeviceInfo + "], SMS Phone [" + syncDevice + "]");

    casdDeviceInfo = udsObject.pemail_address;
    syncDevice = new EmailDevice(user.targetName, "Pager Email");
    syncDevice.isDelete = isEmpty(casdDeviceInfo);
    syncDevice.address = casdDeviceInfo;
    syncDevice.externallyOwned = user.externallyOwned;
    user.devices.add(syncDevice);
    log.debug("pemail_address [" + casdDeviceInfo + "], Pager Email [" + syncDevice + "]");

    casdDeviceInfo = udsObject.phone_number;
    syncDevice = phoneNumberHelper.setVoicePhoneOrPager(user, new VoiceDevice(user.targetName, "Work Phone"), casdDeviceInfo);
    syncDevice.externallyOwned = user.externallyOwned;
    user.devices.add(syncDevice);
    log.debug("phone_number [" + casdDeviceInfo + "], Work Phone [" + syncDevice + "]");

    log.debug("User to sync is: " + user.toString());
    log.info("Exit - makeUserForAddUpdate");
	return user;
}

/**
 * getUserRoles
 *
 * @param caServiceDeskWS
 * @param userObject
 */
function getUserRoles(caServiceDeskWS, userObject)
{
    log.info("Enter - getUserRoles");
	log.info("Searching roles for [" + userObject.userid + "]");

    var userRoles = [];

    var contactHandle = OBJECT_TYPE_CONTACT + ":" + String(userObject.id);
    var whereClause = WHERE_CLAUSE_CONTACT_ROLES.replace("${CONTACT_ID}", String(userObject.id));

    log.info("getUserRoles - contactHandle [" + contactHandle + "], whereClause [" + whereClause + "], attribute [" + ATTRIBUTES_CONTACT_ROLES + "]");

    var udsObjectList = caServiceDeskWS.doSelect(OBJECT_TYPE_CONTACT_ROLES, whereClause, 250, ATTRIBUTES_CONTACT_ROLES);

	udsObjectList.forEach(function(udsObject){
		userRoles.push(udsObject["role_obj.name"]);
	});

    log.info("Found userRoles [" + userRoles + "] for " + userObject.userid);
    log.info("Exit - getUserRoles");
    return userRoles;
} 
