var ticketTypes = ["cr", "iss", "in", "pr", "chg"];
var ticketDesc = ["ticket.requests", "ticket.issues", "ticket.incidents", "ticket.problems", "ticket.changes"];

/**
 * Check to see if the user is logged in. If no sid, log in. A single web
 * services session to CA will be established for a Mobility Session
 * @param sid webservices session id
 * @param username name of the user to login
 * @param password user password
 */
function login(sid, username, password)
{
  log.debug("SID: " + sid);
  if (sid == null || "".equalsIgnoreCase(sid))
  {
    try
    {
      var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);
      caServiceDeskWS.login(username, password);
      sid = caServiceDeskWS.getSessionId(); 
    }
    catch (e)
    {
      if (e.toString().indexOf("HttpHostConnectException") != -1)
      {
        throw new SOAPRequestException("login.connect.errorMessage"); 
      }
      throw new SOAPRequestException("login.errorMessage"); 
    }
  }
  return sid;
}

/**
 * Get the CA Service Desk ID for the selected user
 * @param params collection of parameters used in the request
 * @return XML String containing the SID and User ID
 */
function getHandleForUserId(params)
{
  var resultXml = new StringBuffer("<results>");
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");

  // Create a CA SDM web service session
  sid = login(sid, username, password);

  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);
  
  resultXml.append("<users sid=\"" + caServiceDeskWS.getSessionId() + "\" >");

  // Return the user handle
  resultXml.append("<user>" + caServiceDeskWS.getHandleForUserId(username).substring(4) + "</user>");
  resultXml.append("</users></results>");
  
  log.debug("User Info: " + resultXml.toString());
  return resultXml.toString();
}

/**
 * Retrieve the ticket counts from CA SDM for the given user
 * @param params collection of parameters used in the request
 * @return XML string containing the ticket counts
 */
function getTicketCounts(params)
{
  var resultXml = new StringBuffer("<results><tickets>");
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");
  var whereClause = params.remove("whereClause");

  // Create a CA SDM web service session
  sid = login(sid, username, password);

  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);
  
  // Iterate the ticket types to query SDM and retrieve the counts
  var listHandles = new Array();
  for (var i=0; i<ticketTypes.length; i++)
  {
	  var response = caServiceDeskWS.doQuery(ticketTypes[i], !"cr".equalsIgnoreCase(ticketTypes[i]) ? whereClause : whereClause + " and type = 'R'");
    var ticket = "<ticket type=\"" + ticketTypes[i] + "\" desc=\"" + ticketDesc[i] + "\" count=\"" + response.listLength + "\" />";

    resultXml.append(ticket);
    listHandles[i] = response.listHandle;
  }

  // Free the list handles for this request
  caServiceDeskWS.freeListHandles(listHandles);

  resultXml.append("</tickets></results>");
  return resultXml.toString();
}

/**
 * Frees the list handle from memory in CA SDM
 * @param params collection of parameters used in the request
 */
function freeListHandle(params)
{
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");
  var listHandle = params.remove("listHandle");

  // Create a CA SDM web service session
  sid = login(sid, username, password);

  if (!"".equals(listHandle))
  {
    try
    {
      var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);
      caServiceDeskWS.freeListHandles(new Array(listHandle));
    }
    catch (e)
    {
      log.warn("Unable to free list handle: " + listHandle);
    }
  }
}

/**
 * Retrieve the activity counts from CA SDM for the ticket
 * @param params collection of parameters used in the request
 * @return XML string containing the ticket counts
 */
function getRelatedList(params)
{
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");
  var objectHandle = params.remove("objectHandle");
  var listName = params.remove("listName");

  // Create a CA SDM web service session
  sid = login(sid, username, password);

  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);
  
  // Iterate the ticket types to query SDM and retrieve the counts
  var response = caServiceDeskWS.getRelatedList(objectHandle, listName);
  return response.toString();
}

/**
 * Retrieve the tickets for the specified object type
 * @param params collection of parameters used in the request
 * @return XML string containing the ticket details
 */
function doQuery(params)
{
  var resultXml = new StringBuffer("<results><records>");
  var sid = params.get("sid");
  var username = params.get("username");
  var password = params.get("password");
  var objectType = params.remove("objectType");
  var whereClause = params.remove("whereClause");
  var attributes = params.remove("attributes");
  var startIndex = params.remove("startIndex");
  var chunkSize = params.remove("chunkSize");
  var reverse = params.remove("reverse");
  var listLength = params.remove("listLength");
  var listHandle = params.remove("listHandle");
  
  // Create a CA SDM web service session
  sid = login(sid, username, password);
  
  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);

  try
  {
    var ticketArray = null
    if (startIndex == null || chunkSize == null)
    {
      ticketArray = caServiceDeskWS.processUDSObjectList(caServiceDeskWS.processListValues(objectType, whereClause != null ? whereClause : "", (attributes != null ? attributes.toString().split(",") : [])), processQueryDetails);
    }
    else
    {
      ticketArray = caServiceDeskWS.processUDSObjectList(caServiceDeskWS.processListChunk(objectType, whereClause != null ? whereClause : "", (attributes != null ? attributes.toString().split(",") : []), startIndex, chunkSize, reverse, listHandle, listLength), processQueryDetails);      
    }
    for (var i=0; i<ticketArray.size(); i++)
    {
      resultXml.append(ticketArray.get(i));
    }
  }
  catch (e)
  {
    log.error(e.toString());
    resultXml.append("<faultstring>" + e + "</faultstring>");
  }
  resultXml.append("</records></results>");
  
  return resultXml.toString();
}

/**
 * Process the resultset into a structured XML document that can be
 * processed by tag name
 * @param attributes XML node containing result attributes
 * @return String XML representation of the results
 */
function processQueryDetails(attributes)
{
  var record = new StringBuffer("<record>");
  log.debug("Creating xMatters mobility data from CA SDM data");
  for (var i=0; i<attributes.Attribute.length(); i++)
  {
    var attribute = attributes.Attribute[i];
    this.log.debug("Found record attribute: " + attribute.AttrName + "=" + attribute.AttrValue);
    record.append("<" + attribute.AttrName + ">" + StringEscapeUtils.escapeXml(attribute.AttrValue) + "</" + attribute.AttrName + ">");
  }
  record.append("</record>");
  
  return record;
}

/**
 * Retrieve a list of statuses and their descriptions to translate in the view
 * @param params collection of parameters used in the request
 * @return XML string containing the status details
 */
function getStatuses(params)
{
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");
  var objectType = params.remove("objectType");
  var statusXml = new StringBuffer("<results><statuses>");

  // Create a CA SDM web service session
  sid = login(sid, username, password);

  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);
  var statusMap = caServiceDeskWS.getStatusMap(objectType);
  
  for (var it = statusMap.keySet().iterator(); it.hasNext();)
  {
    var key = it.next();
    var value = statusMap.get(key);
    statusXml.append("<status code=\"" + key + "\">" + value + "</status>");
  }
  statusXml.append("</statuses></results>");
  return statusXml.toString();
}

/**
 * Retrieves the tickets that match the specified criteria
 * @param params collection of parameters used in the request
 * @return XML string containing the matching tickets
 */
function searchTickets(params)
{
  if (params.containsKey("counts"))
  {
    // Create a CA SDM web service session
    var sid = params.get("sid");
    var username = params.get("username");
    var password = params.get("password");
    sid = login(sid, username, password);
  
    var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);
    
    params.put("whereClause", getWhereClause(params, caServiceDeskWS));
    // Iterate the ticket types to query SDM and retrieve the counts
    var response = caServiceDeskWS.doQuery(params.remove("objectType"), params.remove("whereClause"));
    
    return response.toString();
  }
  return doQuery(params);
}

/**
 * Generates a where clause for the query from the given parameters
 * @param params collection of parameters used in the request
 * @return string containing a well formed where clause
 */
function getWhereClause(params, caServiceDeskWS)
{
  var whereClause = new StringBuffer();
  for (var it = params.keySet().iterator(); it.hasNext();)
  {
    var key = it.next();
    if (key.equalsIgnoreCase("ref_num") && !"".equals(params.get(key)) && !"chg".equalsIgnoreCase(params.get("objectType")))
    {
        whereClause.append("ref_num = '" + params.get(key).toString() + "' AND ");
    }
    if (key.equalsIgnoreCase("chg_ref_num") && !"".equals(params.get(key)) && "chg".equalsIgnoreCase(params.get("objectType")))
    {
        whereClause.append("chg_ref_num = '" + params.get(key).toString() + "' AND ");
    }
    if (key.equalsIgnoreCase("requester") && !"".equals(params.get(key)))
    {
      if ("chg".equalsIgnoreCase(params.get("objectType")))
      {
        whereClause.append("requestor = U'" + caServiceDeskWS.getHandleForUserId(params.get(key).toString()).substring(4) + "' AND ");
      }
      else 
      {
        whereClause.append("requested_by = U'" + caServiceDeskWS.getHandleForUserId(params.get(key).toString()).substring(4) + "' AND ");
      }
    }
    if (key.equalsIgnoreCase("status") && !"".equals(params.get(key)))
    {
        whereClause.append("status = '" + params.get(key).toString() + "' AND ");
    }
    if (key.equalsIgnoreCase("impact") && !"".equals(params.get(key)))
    {
        whereClause.append("impact = " + params.get(key).toString() + " AND ");
    }
    if (key.equalsIgnoreCase("urgency") && !"".equals(params.get(key)) && params.get("objectType") != 'chg' && params.get("objectType") != 'iss')
    {
        whereClause.append("urgency = " + params.get(key).toString() + " AND ");
    }
    if (key.equalsIgnoreCase("assignee") && !"".equals(params.get(key)))
    {
        whereClause.append("assignee = U'" + caServiceDeskWS.getHandleForUserId(params.get(key).toString()).substring(4) + "' AND ");
    }
    if (key.equalsIgnoreCase("group") && !"".equals(params.get(key)))
    {
      var attribute = caServiceDeskWS.doSelect("grp", "last_name = '" + params.get(key).toString() + "'", 1, ['id'])..AttrValue;
      
      if (!caServiceDeskWS.isEmpty(params.get(key).toString()) && caServiceDeskWS.isEmpty(attribute))
      {
        throw new SOAPRequestException("Group Name " + params.get("group").toString() + " does NOT exist.");
      }
      whereClause.append("group = U'" + attribute + "' AND ");
    }
    
  }
  
  log.debug("TicketType: " + params.get("objectType"));
  log.debug("WhereClause1: " + whereClause.toString());
  
  if (whereClause.toString().length() > 0)
  {
    if ("cr".equalsIgnoreCase(params.get("objectType"))){
        whereClause += " type = 'R'";
        log.debug("WhereClause2: " + whereClause);
        return whereClause;
    }  
    return whereClause.toString().substring(0, whereClause.toString().length() - 4);  
  }
  throw new SOAPRequestException("ticket.search.empty");
}

/**
 * Retrieve a list of valid transitions and their descriptions to translate in the view
 * @param params collection of parameters used in the request
 * @return XML string containing the transitions details
 */
function getValidTransitions(params)
{
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");
  var objectHandle = params.remove("objectHandle");
  var ticketFactory = params.remove("ticketFactory");
  var transitionXml = new StringBuffer("<results><records>");

  // Create a CA SDM web service session
  sid = login(sid, username, password);

  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);
  var transitionsDoc = caServiceDeskWS.getValidTransitions(objectHandle, ticketFactory);
  
  for (var i=0; i<transitionsDoc.UDSObject.length(); i++)
  {
    transitionXml.append(processQueryDetails(transitionsDoc.UDSObject[i].Attributes));
  }
  
  transitionXml.append("</records></results>");
  return transitionXml;
}

/**
 * Retrieve the tickets for the specified object type
 * @param params collection of parameters used in the request
 * @return XML string containing the ticket details
 */
function getActivities(params)
{
  var resultXml = new StringBuffer("<results><records>");
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");
  var objectType = params.remove("objectType");
  var whereClause = params.remove("whereClause");
  var startIndex = params.remove("startIndex");
  var chunkSize = params.remove("chunkSize");
  var reverse = params.remove("reverse");
  var listLength = params.remove("listLength");
  var listHandle = params.remove("listHandle");
  
  // Create a CA SDM web service session
  sid = login(sid, username, password);
  
  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);

  // Retrieve the correct activity log
  if ("chg".equalsIgnoreCase(objectType))
  {
    objectType = objectType + "alg";
    whereClause = "change_" + whereClause;
  }
  else if ("iss".equalsIgnoreCase(objectType))
  {
    objectType = objectType + "alg";
    whereClause = "issue_" + whereClause;
  }
  else
  {
    objectType = "alg";
    whereClause = "call_req_" + whereClause;
  }

  try
  {
    var recordArray = caServiceDeskWS.processUDSObjectList(caServiceDeskWS.processListChunk(objectType, whereClause != null ? whereClause : "", ["description", "analyst", "last_mod_dt"], startIndex, chunkSize, reverse, listHandle, listLength), processQueryDetails);
    log.debug("activity list size: " + recordArray.size());

    // We need to convert the user ID into a friendly name. To limit the number of webservice calls, we will only look up
    // those users that we haven't already retrieved in this call
    var contactMap = getContactMap(recordArray, caServiceDeskWS);
    for (var i=0; i<recordArray.size(); i++)
    {
      var activityXml = new XML(recordArray.get(i));
      activityXml.combo_name += contactMap.get(activityXml..analyst.toString());
      
      // Format the date to a more friendly format
      var d = new java.util.Date();
      var fmt = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      d.setTime(parseInt(activityXml..last_mod_dt) * 1000);
      activityXml.formatted_date += fmt.format(d);
      
      log.debug("Adding activityXml: " + activityXml.toString());
      resultXml.append(activityXml.toString());
    }
  }
  catch (e)
  {
    log.error(e.toString());
    resultXml.append("<faultstring>" + e + "</faultstring>");
  }
  resultXml.append("</records></results>");
  
  return resultXml.toString();
}

/**
 * Retrieve all of the contacts from the results
 * @param recordArray collection of activity record
 * @param caServiceDeskWS instance of the webservices object
 */
function getContactMap(recordArray, caServiceDeskWS)
{
  var contactList = new StringBuffer();
  var contactMap = new HashMap();

  // Build and array of contacts.
  for (var i=0; i<recordArray.size(); i++)
  {
    if (i > 0)
    {
      contactList.append(" OR ");
    }
    
    var contactXml = new XML(recordArray.get(i));
    var analyst = contactXml..analyst.toString();
    contactList.append("id = U'" + analyst + "'");
  }
  
  // Retrieve the contacts from the management system
  var contactArray = caServiceDeskWS.processUDSObjectList(caServiceDeskWS.processListValues("cnt", contactList.toString(), []), processQueryDetails);
  
  // Iterate the results and add to the Map
  for (var i=0; i<contactArray.size(); i++)
  {
    var contact = new XML(contactArray.get(i));
    var key = contact..id.toString();
    if (!contactMap.containsKey(key))
    {
      contactMap.put(key, contact..combo_name.toString());
    }
  }
  return contactMap;
}



/**
 * Compare the new update with the ticket on the server
 * @param params collection of parameters used in the request
 * @return XML String detailing the differences
 */
function diffTicket(params)
{
  for (var it = params.keySet().iterator(); it.hasNext();)
  {
    var key = it.next();
    log.debug(key + " = " + params.get(key));
  }

  var resultXml = new StringBuffer("<results><records>");
  var oldTicket = new XML(params.remove("oldTicket"));
  var assigneeName = params.remove("assigneeName");
  var groupName = params.remove("groupName");
  var categoryDesc = params.remove("categoryDesc");
  var statusDesc = params.remove("statusDesc");
  var urgencyDesc = params.remove("urgencyDesc");
  var impactDesc = params.remove("impactDesc");
  var newTicket = new XML(doQuery(params));

  // Let's now compare the updateable fields in the two XML docs...if they don't
  // match, add them to a new XML string to be returned to the calling process.
  if (!oldTicket..status.toString().equalsIgnoreCase(newTicket..status.toString()))
  {
    resultXml.append("<record><fieldName>ticket.status</fieldName><updatedValue>" + (!"".equals(statusDesc) ? statusDesc : oldTicket.records.record["status.sym"]) + "</updatedValue><newValue>" + newTicket.records.record["status.sym"] + "</newValue></record>");
  }
  if (oldTicket..urgency != null && !oldTicket..urgency.toString().equalsIgnoreCase(newTicket..urgency.toString()))
  {
    resultXml.append("<record><fieldName>ticket.urgency</fieldName><updatedValue>" + (!"".equals(urgencyDesc) ? urgencyDesc : oldTicket.records.record["urgency.sym"]) + "</updatedValue><newValue>" + newTicket.records.record["urgency.sym"] + "</newValue></record>");
  }
  if (!oldTicket..impact.toString().equalsIgnoreCase(newTicket..impact.toString()))
  {
    resultXml.append("<record><fieldName>ticket.impact</fieldName><updatedValue>" + (!"".equals(impactDesc) ? impactDesc : oldTicket.records.record["impact.sym"]) + "</updatedValue><newValue>" + newTicket.records.record["impact.sym"] + "</newValue></record>");
  }
  if (!oldTicket..category.toString().equalsIgnoreCase(newTicket..category.toString()))
  {
    resultXml.append("<record><fieldName>ticket.category</fieldName><updatedValue>" + (!"".equals(categoryDesc) ? categoryDesc : oldTicket.records.record["category.sym"]) + "</updatedValue><newValue>" + newTicket.records.record["category.sym"] + "</newValue></record>");
  }
  if (!oldTicket..assignee.toString().equalsIgnoreCase(newTicket..assignee.toString()))
  {
    resultXml.append("<record><fieldName>ticket.assignee</fieldName><updatedValue>" + (!"".equals(assigneeName) ? assigneeName : oldTicket.records.record["assignee.combo_name"]) + "</updatedValue><newValue>" + newTicket.records.record["assignee.combo_name"] + "</newValue></record>");
  }
  if (!oldTicket..group.toString().equalsIgnoreCase(newTicket..group.toString()))
  {
    resultXml.append("<record><fieldName>ticket.group</fieldName><updatedValue>" + (!"".equals(groupName) ? groupName : oldTicket.records.record["group.combo_name"]) + "</updatedValue><newValue>" + newTicket.records.record["group.combo_name"] + "</newValue></record>");
  }

  resultXml.append("</records></results>");
  log.debug(resultXml.toString());
  
  return resultXml;
}

/**
 * Updates the ticket in CA SDM
 * @param params collection of parameters used in the request
 */
function updateTicket(params)
{
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");
  var id = params.remove("id");
  var userid = params.remove("userid");
  var status = params.remove("status");
  var objectHandle = params.remove("objectHandle");
  var logType = params.remove("logType");
  var internal = params.remove("internal");
  var description = params.remove("description");
  var status_comment_f = params.get("status_comment_f");

  sid = login(sid, username, password);
  
  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);

  // If a comment is required, add it BEFORE updating the ticket
  if ("1".equalsIgnoreCase(status_comment_f))
  {
    caServiceDeskWS.annotate(objectHandle, "cnt:" + userid, description, logType, internal);
  }
  
  if("chg".equalsIgnoreCase(params.get("ticketType")) || "iss".equalsIgnoreCase(params.get("ticketType"))) {
	  params.remove("urgency");
  }
  
  // We need to convert the autocomplete fields to ids...this is necessary in case the fields
  // were not populated using the autocomplete functionality...so we will always expect to get 
  // values rather than ids in this call...
  var groupName = params.get("group");
  var groupId = caServiceDeskWS.doSelect("grp", "last_name = '" + groupName.toString().replace("'", "''") + "'", 1, ['id'])..AttrValue;
  if (!caServiceDeskWS.isEmpty(groupName.toString()) && caServiceDeskWS.isEmpty(groupId))
  {
    throw new SOAPRequestException("Group Name " + groupName.toString() + " does NOT exist.");
  }
  params.put("group", groupId);
  
  
  var requestedBy = params.get("requested_by");
  if (!caServiceDeskWS.isEmpty(requestedBy.toString()))
  {
    var requesterId = caServiceDeskWS.getHandleForUserId(requestedBy.toString().replace("'", "''"));
    if (caServiceDeskWS.isEmpty(requesterId))
    {
      throw new SOAPRequestException("Requester Name " + requestedBy.toString() + " does NOT exist.");
    }
    params.put("requested_by", requesterId);
  }
  else
  {
    params.remove("requested_by");
  }
  
  var categoryObject = "pcat";
  if("chg".equalsIgnoreCase(params.get("ticketType"))) 
  {
    categoryObject = "chgcat";
  }
  else if ("iss".equalsIgnoreCase(params.get("ticketType"))) 
  {
    categoryObject = "isscat";
  }
  
  var category = params.get("category");
  var categoryId = caServiceDeskWS.doSelect(categoryObject, "sym = '" + category.toString().replace("'", "''") + "'", 1, ['persistent_id'])..AttrValue;
  if (!caServiceDeskWS.isEmpty(category.toString()) && caServiceDeskWS.isEmpty(categoryId))
  {
    throw new SOAPRequestException("Category " + category.toString() + " does NOT exist.");
  }

  params.put("category", categoryId);
  
  params.remove("ticketType");
  
  caServiceDeskWS.updateTicket(id, status, params, objectHandle);
}

/**
 * Creates an activity log entry linked to the current object
 * @param params collection of parameters used in the request
 */
function addActivity(params)
{
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");
  var userid = params.remove("userid");
  var objectHandle = params.remove("objectHandle");
  var logType = params.remove("logType");
  var internal = params.remove("internal");
  var description = params.remove("description");

  sid = login(sid, username, password);
  
  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);

  caServiceDeskWS.annotate(objectHandle, "cnt:" + userid, description, logType, internal);
}

/**
 * Creates an activity log entry linked to the current object
 * @param params collection of parameters used in the request
 */
function createTicket(params)
{
  var resultXml = new StringBuffer("<results><records>");
  var sid = params.remove("sid");
  var username = params.remove("username");
  var password = params.remove("password");
  var userid = params.remove("userid");
  var description = params.remove("description");

  sid = login(sid, username, password);
  
  var caServiceDeskWS = new CAServiceDeskWS(SERVICE_DESK_URL, sid);

  if (caServiceDeskWS.isEmpty(description))
  {
    throw new SOAPRequestException("You must provide a summary for the ticket");
  }

  var response = caServiceDeskWS.createQuickTicket("cnt:" + userid, description);
  var ticketHandle = response.sd::createQuickTicketResponse.newTicketHandle;
  var refNum = response.sd::createQuickTicketResponse.newTicketNumber;
  
  resultXml.append("<record>");

  resultXml.append("<ticketType>" + ticketHandle.substring(0, ticketHandle.indexOf(":")) + "</ticketType>");
  resultXml.append("<objectHandle>" + ticketHandle + "</objectHandle>");
  resultXml.append("<refNum>" + refNum + "</refNum>");

  resultXml.append("</record>");
  resultXml.append("</records></results>");
  
  log.debug(resultXml.toString());
  return resultXml.toString();
}

/**
 * Retrieves the preferred document type according to the default user role 
 * @param collection of query parameters
 */
function getPreferredDocument(params)
{
  var response = new XML(doQuery(params));
  
  /**
   * Odd test to see if an XML node exists.  The most consistent way to test is:
   *    xml.node.length() > 0
   * web searches will find that
   *    xml.node != undefined
   * should also work, but testing has found this is not the case for rhino.
   * Of course the part that is not intuitive is that
   *    xml.node != null
   * will not work!
   */ 
  if (response.records.record.role_obj.length() > 0)
  {
    params.put("objectType", "role");
    params.put("whereClause", "id = " + response.records.record.role_obj);
    response = new XML(doQuery(params));
    
    var preferredDoc = response.records.record.pref_doc;
    log.debug("Preferred Doc: " + preferredDoc);
    log.debug("Ticket Type: " + ticketTypes[preferredDoc - 1]);
    
    if (preferredDoc != null && !"".equals(preferredDoc))
    {
      return ticketTypes[preferredDoc - 1];
    }
  }
  // Default to Incident ticket
  return ticketTypes[2];
}
